# Acknowledgments (and huge thanks) to Cal's A. Guntuboyina
# makre sure you're in the right directory...
# getwd()   setwd("/Users/robotics/Desktop")

#Get the US population dataset 
tt = read.table("USpopFull.R", header = T)
#Get rid of commas in the Population variable
tt$numpop = as.numeric(gsub(",", "", tt$Population))

plot(tt$Year, tt$numpop,  pch = 8,  # type = 'o', # adds lines
     col="blue",
     xlab = "Year(once every ten years)", 
     ylab = "US Population", 
     main = "Population of the United States") 




#Plotting the star brightness data as an example 
# for Frequency Domain Approach

#install.packages("TSA")
library(TSA)
data(star)
plot(star, xlab = "day", ylab = "Brightness", main = "Brightness of a variable star on 600 consecutive nights")


#Plotting Lake Huron Data (Data set available in R)
plot(LakeHuron, main = "Annual Measurements of the level of Lake Huron 1875-1972", xlab = "Year", ylab = "Level in feet", type = "o")

#Plotting US Accidental Deaths Data (Data set available in R)
plot(USAccDeaths, main = "Monthly Accidental Deaths in the US from 1973-1978", xlab = "Time", ylab = "Number of Deaths", type = "o")

# correlogram
ts.obs = rnorm(100)
acf(ts.obs, lag.max = 20, type = "correlation", plot = T, drop.lag.0 = F)



#MA(1) dataset as an example for correlogram
ts = arima.sim(n=100, list(ma = 0.8))
plot(ts, type = "o", main = "Is this data set from a purely random process?", ylab = "Data")
acf(ts, lag.max = 20, type = "correlation", plot = T, drop.lag.0 = F, main = "Correlogram of the Data in the Previous Slide")

#MA(1) dataset with negative theta
ts = arima.sim(n=100, list(ma = - 0.8))
plot(ts, type = "o", main = "Is this data set from a purely random process?", ylab = "Data")
acf(ts, lag.max = 20, type = "correlation", plot = T, drop.lag.0 = F, main = "Correlogram of the Data in the Previous Slide")

#AR(1) dataset
ts = arima.sim(n=100, list(ar =  0.8))
plot(ts, type = "o", main = "Is this data set from a purely random process?", ylab = "Data")
acf(ts, lag.max = 20, type = "correlation", plot = T, drop.lag.0 = F, main = "Correlogram of the Data in the Previous Slide")



# try the autocorrelation of the population data... (for sure!)
acf(tt$numpop, lag.max = 20, type = "correlation", plot = T, drop.lag.0 = F)




# kings of England data
kings <- scan("http://robjhyndman.com/tsdldata/misc/kings.dat",skip=3)

# make a time series
kingstimeseries <- ts(kings)

# show!
kingstimeseries
summary(kingstimeseries)

# plot 
plot.ts(kingstimeseries)

# correlogram
acf(kingstimeseries, lag.max = 20, type = "correlation", 
    plot = T, drop.lag.0 = F, main = "Correlogram of the Kings' age data")


# new york births (by month)
births <- scan("http://robjhyndman.com/tsdldata/data/nybirths.dat")

# tells R that it's by month and give a start year (and 1st month)
birthstimeseries <- ts(births, frequency=12, start=c(1946,1))

# plot it!
plot.ts(birthstimeseries)

# time?
acf(birthstimeseries, lag.max = 20, type = "correlation", 
    plot = T, drop.lag.0 = F, main = "Correlogram of the NYC birth data")



# Monthly souvenir sales...
souvenir <- scan("http://robjhyndman.com/tsdldata/data/fancy.dat")

# 1987-1993, 12 per year
souvenirtimeseries <- ts(souvenir, frequency=12, start=c(1987,1))

# plot it!
plot.ts(souvenirtimeseries)

# time?
acf(souvenirtimeseries, lag.max = 20, type = "correlation", 
    plot = T, drop.lag.0 = F, main = "Correlogram of the souvenir sales data")




# get the TTR package
install.packages("TTR")

# run it
library(TTR)


# moving averages for kings'-age timeseries
# key parameter is n, the moving-average window size
kingstimeseriesSMA3 <- SMA(kingstimeseries,n=3)
plot.ts(kingstimeseriesSMA3)
# show residuals
# take difference and plot it
kingstimeseries - kingstimeseriesSMA3
plot(kingstimeseries - kingstimeseriesSMA3)

# try different n's
kingstimeseriesSMA8 <- SMA(kingstimeseries,n=8)
plot.ts(kingstimeseriesSMA8)

# watch out for the offset inherent in differencing
plot(kingstimeseries - kingstimeseriesSMA8)
diff8 <- kingstimeseries - kingstimeseriesSMA8[4:42]
plot(diff8)



# seasonal analysis
birthstimeseriescomponents <- decompose(birthstimeseries)
plot(birthstimeseriescomponents)
# here is how to extract a component...
birthstimeseriescomponents$trend

# here's how to get a seasonally-adjusted plot:
birthstimeseriesSA <- birthstimeseries - birthstimeseriescomponents$seasonal
plot(birthstimeseriesSA)



# birthday trends
# thanks to www.stanford.edu/~dgleich/notebook/2009/04/birthday_distribution.html
# Read in the data without any processing
data <- read.table('http://www.dartmouth.edu/~chance/teaching_aids/data/birthday.txt',as.is=1)
data[,1] <- as.Date(data[,1],format='%m/%d/%y')
plot(data,xlab='Date',ylab='Live Births',main='Live Births by Date')
# save the plot
png('dob-distribution.png',width=700)
plot(data,xlab='Date',ylab='Live Births',main='Live Births by Date')
dev.off() # write it

# Two groups?
data[,3] <- as.factor(weekdays(data[,1]))
plot(data[,3],data[,2],main='Live Births by Day of Week')
png('dob-dayofweek.png',width=700)
plot(data[,3],data[,2],main='Live Births by Day of Week')
dev.off() # write it



# time-series forecasting

# get rainfall in London data...
rain <- scan("http://robjhyndman.com/tsdldata/hurst/precip1.dat",skip=1)
rainseries <- ts(rain,start=c(1813))
plot.ts(rainseries)

# HoltWinters forecasting (exponential smoothing)
rainseriesforecasts <- HoltWinters(rainseries, beta=FALSE, gamma=FALSE)
plot(rainseriesforecasts)

# you can set the starting level (otherwise the first value gets used)
rainseriesforecasts <- HoltWinters(rainseries, beta=FALSE, gamma=FALSE, l.start=30.56)
plot(rainseriesforecasts)

# get the forecast package...
#install.packages("forecast")
library("forecast")
rainseriesforecasts2 <- forecast.HoltWinters(rainseriesforecasts, h=8)
plot.forecast(rainseriesforecasts2)
acf(rainseriesforecasts2$residuals, lag.max=20, plot = T, drop.lag.0 = F)


# function for plotting forecast errors...
plotForecastErrors <- function(forecasterrors)
{
  # make a histogram of the forecast errors:
  mybinsize <- IQR(forecasterrors)/4
  mysd   <- sd(forecasterrors)
  mymin  <- min(forecasterrors) - mysd*5
  mymax  <- max(forecasterrors) + mysd*3
  # generate normally distributed data with mean 0 and standard deviation mysd
  mynorm <- rnorm(10000, mean=0, sd=mysd)
  mymin2 <- min(mynorm)
  mymax2 <- max(mynorm)
  if (mymin2 < mymin) { mymin <- mymin2 }
  if (mymax2 > mymax) { mymax <- mymax2 }
  # make a red histogram of the forecast errors, with the normally distributed data overlaid:
  mybins <- seq(mymin, mymax, mybinsize)
  hist(forecasterrors, col="red", freq=FALSE, breaks=mybins)
  # freq=FALSE ensures the area under the histogram = 1
  # generate normally distributed data with mean 0 and standard deviation mysd
  myhist <- hist(mynorm, plot=FALSE, breaks=mybins)
  # plot the normal curve as a blue line on top of the histogram of forecast errors:
  points(myhist$mids, myhist$density, type="l", col="blue", lwd=2)
}

# now plot them...
plotForecastErrors(rainseriesforecasts2$residuals)



# here are the skirt-hem-diameter data...
skirts <- scan("http://robjhyndman.com/tsdldata/roberts/skirts.dat",skip=5)
skirtsseries <- ts(skirts,start=c(1866))
plot.ts(skirtsseries,main="Skirt-hem diameters vs. year")
# now we allow it to estimate alpha and beta: smoothing and trend-following
skirtsseriesforecasts <- HoltWinters(skirtsseries, gamma=FALSE)
skirtsseriesforecasts
plot(skirtsseriesforecasts)
skirtsseriesforecasts2 <- forecast.HoltWinters(skirtsseriesforecasts, h=19)
plot.forecast(skirtsseriesforecasts2)

acf(skirtsseriesforecasts2$residuals, lag.max=20, plot = T, drop.lag.0 = F)
# check the p-value... using the "Ljung-Box test"
Box.test(skirtsseriesforecasts2$residuals, lag=20, type="Ljung-Box")
# p ~ .47 so no trong evidence for more signal here...
plot.ts(skirtsseriesforecasts2$residuals)            # make a time plot
plotForecastErrors(skirtsseriesforecasts2$residuals) # make a histogram


# souvenir data...
plot(souvenirtimeseries)
# does not seem additive... transform it!
logsouvenirtimeseries <- log(souvenirtimeseries)
plot(logsouvenirtimeseries)

# model with smoothing, trend, AND seasonal variation...
souvenirtimeseriesforecasts <- HoltWinters(logsouvenirtimeseries)
souvenirtimeseriesforecasts
plot(souvenirtimeseriesforecasts)
souvenirtimeseriesforecasts2 <- forecast.HoltWinters(souvenirtimeseriesforecasts, 
                                                     h=48)
plot.forecast(souvenirtimeseriesforecasts2)
# check residual correlations (not much there)
acf(souvenirtimeseriesforecasts2$residuals, lag.max=20, plot = T, drop.lag.0 = F)
# and the Ljung-Box test says p > 0.5, ok, then...
Box.test(souvenirtimeseriesforecasts2$residuals, lag=20, type="Ljung-Box")
# plot the residuals
plot.ts(souvenirtimeseriesforecasts2$residuals)            # make a time plot
# and their histogram...
plotForecastErrors(souvenirtimeseriesforecasts2$residuals) # make a histogram






# image scatter plot
# Drawing a scatter plot of raster images
doInstall <- TRUE  # Change to FALSE if you don't want packages installed.
toInstall <- c("png", "devtools", "MASS", "RCurl")
if(doInstall){install.packages(toInstall, repos = "http://cran.r-project.org")}
lapply(toInstall, library, character.only = TRUE)

# Some helper functions, lineFinder and makeTable
source_gist("818983")
source_gist("818986")

# In as few lines as possible, get URLs for .PNGs of each flag
importHTML <- readLines("http://en.wikipedia.org/wiki/World_flags")
importHTML[lineFinder("thumbborder", importHTML)]
pngURLs <- makeTable(makeTable(importHTML[lineFinder("thumbborder",
                                                     importHTML)], "src=\"//")[, 2], "\" width=\"")[, 1]
pngURLs <- paste0("http://", pngURLs)

# CAUTION: The following loop will download 204 .PNG images
# of flags from Wikipedia. Please be considerate, and don't run
# this part of the script any more than you need to.
pngList <- list()
for(ii in 1:length(pngURLs)){
  tempName <- paste("Flag", ii)
  tempPNG <- readPNG(getURLContent(pngURLs[ii]))  # Downloads & loads PNGs
  pngList[[tempName]] <- tempPNG  # And assigns them to a list.
}

# Very simple dimension reduction -- just the mean R, G, and B values
meanRGB <- t(sapply(pngList, function(ll){
  apply(ll[, , -4], 3, mean)
}))

# The dimensions of each item are equal to the pixel dimensions of the .PNG
flagDimensions <- t(sapply(pngList, function(ll){
  dim(ll)[1:2]
}))

# Similarity, through Kruskal non-metric MDS
flagDistance <- dist(meanRGB)
flagDistance[flagDistance <= 0] <- 1e-10

MDS <- isoMDS(flagDistance)$points
plot(MDS, col = rgb(meanRGB), pch = 20, cex = 2)

# Plot:
boxParameter <- 5000  #6000  # To alter dimensions of raster image bounding box
par(bg = gray(8/9))
plot(MDS, type = "n", asp = 1)
for(ii in 1:length(pngList)){  # Go through each flag
  tempName <- rownames(MDS)[ii]
  Coords <- MDS[tempName, 1:2]  # Get coordinates from MDS
  Dims <- flagDimensions[tempName, ]  # Get pixel dimensions
  rasterImage(pngList[[tempName]],  # Plot each flag with these boundaries:
              Coords[1]-Dims[2]/boxParameter, Coords[2]-Dims[1]/boxParameter,
              Coords[1]+Dims[2]/boxParameter, Coords[2]+Dims[1]/boxParameter)
}